# RPM specfile for the LetsEncryptManage package
# Copyright 2020 Parliament Hill Computers Ltd, all rights reserved.
# Author: ADDW March 2020
# SCCS: @(#)pV	1.18 12/23/20 19:55:48

%define name LetsEncryptManage
# Define the version number, etc.
# These come from VERSION_FILE, but that will not be usable very early on
# when it is in a tar file. Thus this .spec file is built from a template.
# Beware: several things start VERSION:
%define version 1.17
%define release 6.6

# Users that we create
%define	acmeuser acme
%define	rsauser  rsa
%define	acme_dir /var/www/acme/
%define	bindir %{acme_dir}/bin

Summary: Simple & Secure Maintainance of Let's Encrypt Certificates
Name: %{name}
Version: %{version}
Release: %{release}
Source: http://www.phcomp.co.uk/Downloads/%{name}-%{version}-%{release}.tar.gz
Vendor: %{vendor}
URL: %{phcl_url}

# Documentation directory.
# This changed with Fedora 30 (or something) and RedHat/CentOS 8
# It used to be /usr/share/doc/package-version, went to /usr/share/doc/package
%if 0%{?fedora} >= 30 || 0%{?rhel} >= 8
%define docd %{_docdir}/%{name}
%else
%define docd %{_docdir}/%{name}-%{version}
%endif

BuildArchitectures: noarch

License: GPL 3 or later

Packager: %{packager}

Group: Applications/Archiving
BuildRoot: %{_builddir}/%{name}-root
%if 0%{?rhel} >= 8
Requires: ksh python3 openssl curl coreutils sed cronie wget grep webserver
%else
Requires: ksh python  openssl curl coreutils sed cronie wget grep webserver
%endif
Requires(pre): shadow-utils

%pre

# SELinux can screw adding the user rsa.
# This is because its $HOME is under /var/www/ - for some reason that is not considered a good
# place for home directories.
# See: https://forums.fedoraforum.org/showthread.php?275621-SElinux-denies-creating-home-dir
# Don't fiddle with their SELinux setup - create the directory by hand and not via useradd.
hand_made=0
[[ -d %{acme_dir} ]] || hand_made=1
[[ $hand_made = 0 ]] || mkdir %{acme_dir} || exit

# Create the 2 users:
getent passwd %{rsauser} >/dev/null || \
    useradd -c "RSA key generator"     --no-create-home --home-dir %{acme_dir} --user-group     --inactive 0 %{rsauser} || exit

getent passwd %{acmeuser} >/dev/null || \
    useradd -c "Let's Encrypt Renewer" --no-create-home --home-dir %{acme_dir} --gid %{rsauser} --inactive 0 %{acmeuser} || exit

# If hand made set the user/group ownership:
[[ $hand_made = 0 ]] || chown %{rsauser}.%{rsauser} %{acme_dir} || exit

exit 0

# This is only needed if we want to build a relocatable RPM:
#Prefix: %%{_prefix}

%description
Simple, thus auditable, scripts to get and maintain Let's Encrypt certificates
These free certificates let you serve up https:// web pages.

Scripts to:
* Generate Let's Encrypt account key
* Generate RSA private keys
* Contact Let's Encrypt signed key
* Get a new signed key once the current one is 60 days old
Signed key can contain Let's Encrypt chain certificate

The scripts are installed in %{acme_dir}

You will still need to configure your web server to use the certificates.
Help is given for this. There are snippets for Nginx and Apache.

The communication wih Let's Encrypt is done using acme_tiny.py, from here:
https://github.com/diafygi/acme-tiny

See:
https://www.phcomp.co.uk/Tutorials/Web-Technologies/Configure-Lets-Encrypt-with-acme_tiny.html
%prep
%setup -q

%build

%install
# RPM_BUILD_ROOT is set by BuildRoot: above

rm -rf $RPM_BUILD_ROOT
mkdir -p $RPM_BUILD_ROOT/%{acme_dir}

echo %{acmeuser} > $RPM_BUILD_ROOT/%{acme_dir}ACMEUSER
echo %{rsauser}  > $RPM_BUILD_ROOT/%{acme_dir}RSAUSER

make DESTDIR=$RPM_BUILD_ROOT%{acme_dir} VERSION=%{version} RELEASE=%{release} DOCDIR=$RPM_BUILD_ROOT/%{docd} install

%clean
rm -rf $RPM_BUILD_ROOT

%post

# Do some of what happens in InitialSetup

# Find the group name of Apache workers (or whatever web server).
# Test somewhat more than found on RedHat/CentOS
# This might get the wrong answer if several web servers are installed,
# could, maybe, probe further with: ps -fu xxx
ApacheUser=
for id in apache nginx lighttpd www www-data wwwrun
do	if id $id >/dev/null 2>&1
	then	ApacheUser=$id
		break
	fi
done

# Should not happen - unless the web server is not one of those above
if [[ -z $ApacheUser ]]
then	echo "I cannot see that a web server is installed" >&2
	exit 2
fi

# What group does that run as ?
ApacheGroup=$( id --group --name $ApacheUser )

cd %{acme_dir}

# As RedHat runs the different web servers as different users this can only
# be done at install time. This can break rpm --verify

chown %{acmeuser}:$ApacheGroup %{acme_dir}/acme-challenge/
: CHECK mode 2711

chgrp $ApacheGroup %{acme_dir}RSAprivateKeys %{acme_dir}SignedCertificates %{acme_dir}tmp
: CHECK mode 2710

# Generate a sample crontab.
# Randomise the time so that not everyone hits Let's Encrypt at the same time:
(( hour = RANDOM % 24 ))
(( hour2 = (hour + 12) % 24 ))
(( minute = RANDOM % 60 ))

shell=$SHELL	# default hopefully vaguely sensible
# Try for something better
type bash > /dev/null 2>&1 && shell=bash
type ksh  > /dev/null 2>&1 && shell=ksh
[[ -z $shell ]]            && shell=sh

sed -e s/MIN/$minute/ -e s/HOUR1/$hour/ -e s/HOUR2/$hour2/ -e "s@SHELL@$shell@" %{acme_dir}Templates/Renewal.crontab.in > %{acme_dir}Renewal.crontab

# Snippets to put into the http/nginx config:
for snip in httpd.conf.challenge vhosts-port-80-and-443.conf nginx.conf.add-ssl
do	sed -e "s@ACMEHOME@%{acme_dir}@" %{acme_dir}Templates/$snip.in > %{acme_dir}$snip
done

exit 0

%files
%defattr(-,root,root)
%dir %{bindir}
# acme_tiny.py is not made executable, it does not need it - see GetSignedCertificate & RenewCertificates
# Also the #! line is removed due to latest rpmbuild winge as the #! line uses 'env' - which is no, no these days,
# it also does not specify which of python2 or python3 - which is also no, no.
%attr(444,  root,  root ) %{bindir}/acme_tiny.py

%attr(555,  root,  root ) %{bindir}/CheckSiteAccess
%attr(555,  root,  root ) %{bindir}/CreateSigningRequests
%attr(555,  root,  root ) %{bindir}/GetSignedCertificate
%attr(555,  root,  root ) %{bindir}/InitialSetup
%attr(555,  root,  root ) %{bindir}/RenewCertificates
%attr(555,  root,  root ) %{bindir}/ShowSSLinfo

# rsa user's home:
%dir %attr(751, %{rsauser},  %{rsauser})  %{acme_dir}

# The web server needs to be able to see the file below, so directory set-gid
# the web server is needed - the group ownership is corrected in the %post script
# above
%dir %attr(2711, %{acmeuser}, root)       %{acme_dir}/acme-challenge
     %attr(444,  root,        root)       %{acme_dir}/acme-challenge/index.html

# Do not let the web server see anything else

%dir %attr(710,  %{rsauser},  %{rsauser}) %{acme_dir}CertificateSigningRequests

# This must be specified as %dir separately else get error with EXAMPLE.cnf 'listed twice'
%dir %attr(700,  %{rsauser},  %{rsauser}) %{acme_dir}SSLConfigs
     %attr(640,  %{rsauser},  %{rsauser}) %{acme_dir}SSLConfigs/EXAMPLE.cnf

%dir %attr(2710, %{rsauser},  %{rsauser}) %{acme_dir}RSAprivateKeys
%dir %attr(700,  %{acmeuser}, root)       %{acme_dir}LetsEncryptAccounts
%dir %attr(2710, %{acmeuser}, root)       %{acme_dir}SignedCertificates
%dir %attr(775,  %{acmeuser}, root)       %{acme_dir}web-server-status
%dir %attr(2710, %{acmeuser}, root)       %{acme_dir}tmp
     %attr(660,  %{rsauser},  %{rsauser}) %{acme_dir}/.rnd
     %attr(444,  root,        root)       %{acme_dir}/ACMEUSER
     %attr(444,  root,        root)       %{acme_dir}/RSAUSER

%config(noreplace) %attr(644,  %{acmeuser}, root)       %{acme_dir}/Renewals.list

%{acme_dir}/README
%{acme_dir}/VERSION_FILE
%{acme_dir}/Hacks
%{acme_dir}/TODO
%{acme_dir}/License
%{acme_dir}/SignedCertificates/README
%{acme_dir}/web-server-status/README
%{acme_dir}/Contributors
%{acme_dir}/GenericSSL.conf
%{acme_dir}/httpd.conf.add-ssl
%{acme_dir}/lets-encrypt-r3.pem
%{acme_dir}/ScriptToRestartWebServer

# these need processing on install
%dir %{acme_dir}/Templates
%{acme_dir}/Templates/httpd.conf.challenge.in
%{acme_dir}/Templates/Renewal.crontab.in
%{acme_dir}/Templates/vhosts-port-80-and-443.conf.in
%{acme_dir}/Templates/nginx.conf.add-ssl.in

%doc TODO Hacks License Contributors README

%changelog
* Wed Dec 23 2020 Alain Williams <addw@phcomp.co.uk>
- Cope with openssl sometimes putting spaces around CN=
* Tue Dec 22 2020 Alain Williams <addw@phcomp.co.uk>
- Increase the warning time on cross certificate expiry to 180 days
* Tue Dec 15 2020 Alain Williams <addw@phcomp.co.uk>
- New cross sign certificate: lets-encrypt-r3.pem
* Tue Jul 21 2020 Alain Williams <addw@phcomp.co.uk>
- acme_tiny.py remove even commented #! on the first line
* Mon Jul 20 2020 Alain Williams <addw@phcomp.co.uk>
- acme_tiny.py no longer executable
* Sun Jul 19 2020 Alain Williams <addw@phcomp.co.uk>
- Contents of bin/ directory were not being made executable
* Sun Jul 12 2020 Alain Williams <addw@phcomp.co.uk>
- New web-server-status/ & ScriptToRestartWebServer
* Tue May 05 2020 Alain Williams <addw@phcomp.co.uk>
- Set umask to 022 in RenewCertificates & GetSignedCertificate
- .spec file: Ownership of acme-challenge was set wrong, not set groupid
- and give good value for ACMEHOME subscitution
- Apache examples: move to 2.4 syntax, 2.2 syntax there but commented out.
* Tue Apr 14 2020 Alain Williams <addw@phcomp.co.uk>
- Change package name to LetsEncryptManage
- Change license to GPL 3 or later
- Version now held in VERSION_FILE
- A bit more flexibility: nginx lighttpd
- Process an nginx template file
- Scripts will always process a --help option
- ShowSSLinfo: Add --family & --all-families options
- Various bug fixes & better comments
- Family names restricted to RE: ^[-a-zA-Z0-9_.]+$ & not start with a -
* Sat Apr 04 2020 Alain Williams <addw@phcomp.co.uk>
- Initial version to RPM
